/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-31 09:28:03.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ObjectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.util.AsyncExportConcurrentUtil;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.examing.ExamBoss;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 考试分数相关接口
 *
 * @author: QMTH
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试分数相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/score")
public class ExamScoreController extends ControllerSupport {

    @Autowired
    private AsyncExportService asyncExportService;

    @Autowired
    private ExamScoreService examScoreService;

    @Autowired
    private RedisClient redisClient;

    @PostMapping("/statistic/list")
    @ApiOperation(value = "查询“成绩统计”列表(分页)")
    public Page<ExamScoreInfo> getExamAuditList(@RequestBody ExamScoreQuery query) {
        Page<ExamScoreInfo> examScoreList = examScoreService.getExamScoreList(query);

        examScoreList.getContent().forEach(p -> {
            p.setIdentityNumber(IdentityNumberHelper.conceal(getRootOrgId(), p.getIdentityNumber()));
        });
        return examScoreList;
    }

    @Naked
    @GetMapping("/statistic/list/export")
    @ApiOperation(value = "导出“成绩统计”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamRecordDetailList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamScoreQuery newQuery = new JsonMapper().parseJson(query, ExamScoreQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");
        List<ExamScoreInfo> examScoreList = examScoreService.exportExamScoreList(newQuery);
        ExportUtils.exportEXCEL("成绩统计列表", ExamScoreInfo.class, examScoreList, response);
    }

    @GetMapping("/statistic/list/export/async")
    @ApiOperation(value = "导出“成绩统计”列表(异步)",
            notes = "参数示例：query={\"rootOrgId\":0,\"examId\":1,\"startLimit\":1,\"endLimit\":1}")
    public void exportExamScores(@RequestParam String query) {
    	User user=getAccessUser();
        asyncExportService.exportExamScoreStatistics(query,user);
        AsyncExportConcurrentUtil.checkOrStartJob();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "成绩统计-导出","导出条件:"+query));
    }

    @ApiOperation(value = "根据examStudentId获取客观分信息")
    @GetMapping("/queryObjectiveScoreList")
    public List<ObjectiveScoreInfo> queryObjectiveScoreList(@RequestParam Long examStudentId) {
        Check.isNull(examStudentId, "examStudentId 不能为空");

        List<ObjectiveScoreInfo> resultList = examScoreService.queryObjectiveScoreList(examStudentId);


        //如果有未处理完成的考试记录，需要将未处理的考试记录数据添加到列表中
        String examBossKey = RedisKeyHelper.getBuilder().examBossKey(examStudentId);
        ExamBoss examBoss = redisClient.get(examBossKey, ExamBoss.class);
        if (null != examBoss) {
            //未完全同步的考试记录id
            List<Long> unSyncedExamRecordDataIds = examBoss.getExamRecordDataIds();
            if (null != unSyncedExamRecordDataIds && !unSyncedExamRecordDataIds.isEmpty()) {
                //正序排列
                unSyncedExamRecordDataIds.sort(Long::compareTo);

                //已考次数
                Integer examUsedNum = 0;

                for (Long examRecordDataId : unSyncedExamRecordDataIds) {
                    ExamRecordData examRecordData =
                            redisClient.get(RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId),
                                    ExamRecordData.class);
                    if (null == examRecordData) {
                        throw new StatusException("100001", "考试记录的缓存数据有误");
                    }

                    //考试中的数据不展示
                    if (ExamRecordStatus.EXAM_ING == examRecordData.getExamRecordStatus()) {
                        continue;
                    }

                    if (!resultList.isEmpty()) {
                        examUsedNum = resultList.get(resultList.size() - 1).getExamOrder();
                    }

                    ObjectiveScoreInfo cachedObjectiveScoreInfo = getCachedObjectiveScoreInfo(examRecordData);
                    cachedObjectiveScoreInfo.setExamOrder(
                            getExamOrder(examRecordData.getExamId(), examRecordData.getStudentId(), examUsedNum));

                    resultList.add(cachedObjectiveScoreInfo);
                }
            }
        }

        return resultList;
    }

    private ObjectiveScoreInfo getCachedObjectiveScoreInfo(final ExamRecordData examRecordData) {


        ObjectiveScoreInfo objectiveScoreInfo = new ObjectiveScoreInfo();
        objectiveScoreInfo.setExamRecordDataId(examRecordData.getId());
        objectiveScoreInfo.setStartTime(examRecordData.getStartTime());
        objectiveScoreInfo.setEndTime(examRecordData.getEndTime());

        //如果考试没有结束，则只能返回部分数据
        if (!isExamRecordEnded(examRecordData)) {
            objectiveScoreInfo.setIsExamEnded(false);
            return objectiveScoreInfo;
        } else {
            objectiveScoreInfo.setIsExamEnded(true);
        }

        if (examRecordData.getIsIllegality() == null || !examRecordData.getIsIllegality()) {
            if ((null != examRecordData.getIsWarn() && !examRecordData.getIsWarn())
                    || (null != examRecordData.getIsWarn() && examRecordData.getIsWarn()
                    && null != examRecordData.getIsAudit() && examRecordData.getIsAudit())) {
                objectiveScoreInfo.setIsAuditing(true);

                //缓存中的分数是存储在临时考试记录表中的，所以需要从考试记录缓存中取
                objectiveScoreInfo.setObjectiveScore(examRecordData.getObjectiveScore());
            } else {
                objectiveScoreInfo.setIsAuditing(false);
            }

            objectiveScoreInfo.setIsIllegality(false);
        } else {
            objectiveScoreInfo.setIsIllegality(true);
        }

        return objectiveScoreInfo;
    }

    private boolean isExamRecordEnded(ExamRecordData examRecordData) {
        //如果考试记录状态为已处理，则直接返回true.
        if (examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_END ||
                examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_OVERDUE ||
                examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_INVALID) {
            return true;
        }
        return false;
    }

    /**
     * 计算考试次数
     *
     * @param examId      考试id
     * @param studentId   学生id
     * @param usedExamNum 已考次数
     * @return
     */
    private Integer getExamOrder(Long examId, Long studentId, Integer usedExamNum) {
        ExamSettingsCacheBean cachedExam = ExamCacheTransferHelper.getDefaultCachedExam(examId);
        Integer canExamTimes = cachedExam.getExamTimes() == null ? 0 : cachedExam.getExamTimes().intValue();//可考次数

        //超过可考次数,始终为可考次数+1
        if (usedExamNum > canExamTimes) {
            return canExamTimes;
        }

        return usedExamNum;

    }

}
