package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.MD5;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.OfflineExamService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.OfflineExamCourseInfo;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;


/**
 * @author chenken
 * @date 2018年9月5日 下午3:33:26
 * @company QMTH
 * @description 离线考试Controller
 */
@Api(tags = "离线考试控制")
@RestController
@RequestMapping("${$rmp.ctr.oe}/offlineExam")
public class OfflineExamController extends ControllerSupport {
	private static final Logger log = LoggerFactory.getLogger(OfflineExamController.class);
    @Autowired
    private OfflineExamService offlineExamService;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    SystemProperties systemConfig;
    /**
     * 答案文件最大限制
     * 单位:M
     */
    private static int answerMaxsize = 30;

    public static final String TEMP_FILE_EXP = "offlineExam/";


    /**
     * 获取离线考试列表
     *
     * @return
     */
    @ApiOperation(value = "获取离线考试列表")
    @GetMapping("/getOfflineCourse")
    public List<OfflineExamCourseInfo> getOfflineCourse() {
        User user = getAccessUser();
        List<OfflineExamCourseInfo> offlineExamCourseInfos = offlineExamService.getOfflineCourse(user.getUserId());
        return offlineExamCourseInfos;
    }

    /**
     * 开始考试
     *
     * @param examStudentId
     */
    @ApiOperation(value = "离线考试:开始考试")
    @GetMapping("/startOfflineExam")
    public void startOfflineExam(@RequestParam long examStudentId) {
        Check.isNull(examStudentId, "examStudentId不能为空");

        boolean isEnable = examStudentService.isEnableExamStudent(examStudentId);
        if(!isEnable){
            throw new StatusException("000500", "当前考生已禁用！");
        }

        offlineExamService.startOfflineExam(examStudentId);
    }

    /**
     * 交卷
     */
    @ApiOperation(value = "离线考试:交卷")
    @PostMapping("/submitPaper")
    public void submitPaper(@RequestParam(value = "file") MultipartFile file,
                            @RequestParam long examRecordDataId) throws Exception {
        Check.isNull(file, "file不能为空");
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");

        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordData.getExamType() != ExamType.OFFLINE) {
            throw new StatusException("OfflineExamController-submitPaper-002", "非离线考试");
        }

        boolean isEnable = examStudentService.isEnableExamStudent(examRecordData.getExamStudentId());
        if(!isEnable){
            throw new StatusException("000500", "当前考生已禁用！");
        }

        String fileName = file.getOriginalFilename();
        int index = fileName.lastIndexOf(".");
        String fileSuffix = fileName.substring(index + 1, fileName.length()).toUpperCase();

        if (!"PDF".equals(fileSuffix) && !"ZIP".equals(fileSuffix)
                && !"JPG".equals(fileSuffix) && !"JPEG".equals(fileSuffix) && !"PNG".equals(fileSuffix)) {
            throw new StatusException("OfflineExamController-submitPaper-001", "文件格式不正确");
        }

        String offlineUploadFileType = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                examRecordData.getStudentId(),
                ExamProperties.OFFLINE_UPLOAD_FILE_TYPE.name()).getValue();
        if (StringUtils.isBlank(offlineUploadFileType) || "[]".equals(offlineUploadFileType)) {
            throw new StatusException("OfflineExamController-submitPaper-003", "当前考试设置不允许上传附件");
        }
        if (offlineUploadFileType.indexOf(fileSuffix) < 0) {
            throw new StatusException("OfflineExamController-submitPaper-004", "当前考试允许上传文件格式为:" + offlineUploadFileType);
        }
        //判断文件大小
        long fileSize = file.getSize();

        if (fileSize > answerMaxsize * 1048576) {
            throw new StatusException("OfflineExamController-submitPaper-005", "文件大小不能超过" + answerMaxsize + "M");
        }

        File f = getUploadFile(file);

        offlineExamService.submitPaper(examRecordDataId, f,fileSuffix);

        User user=getAccessUser();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "考试进度详情-上传作答","考试记录ID:"+examRecordDataId+" 考生ID:"+examRecordData.getExamStudentId()));
    }

    /**
     * 交卷
     *
     * @param fileArray            文件数组
     * @param examRecordDataId 考试记录id
     * @param fileMd5Array        文件摘要信息
     * @throws Exception
     */
    @ApiOperation(value = "离线考试:多文件交卷")
    @PostMapping("/batchSubmitPaper")
    public void batchSubmitPaper(@ApiParam(value = "文件数组") @RequestParam MultipartFile[] fileArray,
                                 @ApiParam(value = "考试记录id") @RequestParam long examRecordDataId,
                                 @ApiParam(value = "文件类型:ZIP/PDF/IMAGE") @RequestParam String fileType,
                                 @ApiParam(value = "md5加密的文件摘要") @RequestParam String[] fileMd5Array) throws Exception {
        Check.isNull(fileArray, "files不能为空");
        Check.isNull(examRecordDataId, "examRecordDataId不能为空");
        if (fileArray.length != fileMd5Array.length) {
            throw new StatusException("200001", "文件数量和文件摘要信息不匹配");
        }

        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordData.getExamType() != ExamType.OFFLINE) {
            throw new StatusException("100003", "非离线考试");
        }

        boolean isEnable = examStudentService.isEnableExamStudent(examRecordData.getExamStudentId());
        if(!isEnable){
            throw new StatusException("000500", "当前考生已禁用！");
        }

        String offlineUploadFileType = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                examRecordData.getStudentId(),
                ExamProperties.OFFLINE_UPLOAD_FILE_TYPE.name()).getValue();
        if (StringUtils.isBlank(offlineUploadFileType) || "[]".equals(offlineUploadFileType)) {
            throw new StatusException("OfflineExamController-submitPaper-003", "当前考试设置不允许上传附件");
        }

        String upperFileType=fileType.toUpperCase();
        if (("PDF".equals(upperFileType) || "ZIP".equals(upperFileType))&&fileArray.length>1){
        	StringBuilder sb=new StringBuilder();
        	for (int i = 0; i < fileArray.length; i++) {
                MultipartFile file = fileArray[i];
                String fileName = file.getOriginalFilename();
                sb.append(fileName).append(",");
        	}
        	sb.deleteCharAt(sb.length()-1);
        	log.error("batchSubmitPaper-duplicate:"+examRecordDataId+":"+sb);
        	throw new StatusException("000500", "上传"+upperFileType+"类型文件时只能传一个文件");
        }
        
        List<File> resultFiles=new ArrayList<>();
        for (int i = 0; i < fileArray.length; i++) {
            MultipartFile file = fileArray[i];
            String fileName = file.getOriginalFilename();
            int index = fileName.lastIndexOf(".");
            String fileSuffix = fileName.substring(index + 1, fileName.length()).toUpperCase();

            if (!"PDF".equals(upperFileType) && !"ZIP".equals(upperFileType)
                    && !"IMAGE".equals(upperFileType)) {
                throw new StatusException("100001", "文件类型不正确");
            }

            if (!"PDF".equals(fileSuffix) && !"ZIP".equals(fileSuffix)
                    && !"JPG".equals(fileSuffix) && !"JPEG".equals(fileSuffix) && !"PNG".equals(fileSuffix)) {
                throw new StatusException("100002", "文件格式不正确");
            }



            //图片类型特殊处理
            if (fileSuffix.equals("JPG") || fileSuffix.equals("JPEG") || fileSuffix.equals("PNG")) {
                if (offlineUploadFileType.indexOf("IMAGE")<0) {
                    throw new StatusException("100004", "当前考试允许上传文件格式为:" + offlineUploadFileType);
                }
            }else{
                if (offlineUploadFileType.indexOf(fileSuffix) < 0) {
                    throw new StatusException("OfflineExamController-submitPaper-004", "当前考试允许上传文件格式为:" + offlineUploadFileType);
                }
            }

            //判断文件大小
            long fileSize = file.getSize();

            if (fileSize > answerMaxsize * 1048576) {
                throw new StatusException("OfflineExamController-submitPaper-005", "文件大小不能超过" + answerMaxsize + "M");
            }

            File f = getUploadFile(file);
            String strMd5 = MD5.md5Hex(f);
            String summary = fileMd5Array[i];
            if (!strMd5.equals(summary)) {
                throw new StatusException("OfflineExamController-submitPaper-006", "文件数据不完整，请重新上传");
            }

            resultFiles.add(f);
        }

        offlineExamService.batchSubmitPaper(examRecordDataId,resultFiles,fileType);

        //最后删除临时文件
        for (File tempFile : resultFiles) {
            tempFile.delete();
        }
        User user=getAccessUser();
        ReportsUtil.report(new AdminOperateReport(user.getRootOrgId(), user.getUserId(), "考试进度详情-上传作答","考试记录ID:"+examRecordDataId+" 考生ID:"+examRecordData.getExamStudentId()));
    }

    private File getUploadFile(MultipartFile file) {
        //建临时文件夹
        File dirFile = new File(TEMP_FILE_EXP);
        if (!dirFile.exists()) {
            dirFile.mkdirs();
        }
        String fileName = file.getOriginalFilename();
        File tempFile = new File(TEMP_FILE_EXP + fileName);
        OutputStream os = null;
        try {
            os = new FileOutputStream(tempFile);
            IOUtils.copyLarge(file.getInputStream(), os);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            IOUtils.closeQuietly(os);
        }
        return tempFile;
    }
}
