package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.*;
import cn.com.qmth.examcloud.core.oe.admin.api.request.*;
import cn.com.qmth.examcloud.core.oe.admin.api.response.*;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.*;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.*;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordForMarkingService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.question.commons.core.question.AnswerType;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionStructure;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionUnit;
import cn.com.qmth.examcloud.question.commons.core.question.QuestionType;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.QuestionCacheBean;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author chenken
 * @date 2018年9月19日 下午2:44:12
 * @company QMTH
 * @description ExamRecordCloudServiceProvider.java
 */
@Api(tags = "考试记录相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/examRecord")
public class ExamRecordCloudServiceProvider extends ControllerSupport implements ExamRecordCloudService {

    /**
     *
     */
    private static final long serialVersionUID = 7678785677176906341L;

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamCaptureRepo examCaptureRepo;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamRecordForMarkingService examRecordForMarkingService;

    @Autowired
    private ExamRecordQuestionsRepo examRecordQuestionsRepo;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamRecordDataService examRecordDataService;


    @Override
    @ApiOperation(value = "查询是否已经开考")
    @PostMapping("/checkExamIsStarted")
    public CheckExamIsStartedResp checkExamIsStarted(@RequestBody CheckExamIsStartedReq req) {
        Check.isNull(req.getExamId(), "examId不能为空");

        Specification<ExamRecordDataEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("examId"), req.getExamId()));

            if (req.getCourseId() != null) {
                predicates.add(cb.equal(root.get("courseId"), req.getCourseId()));
            }

            if (req.getStudentId() != null) {
                predicates.add(cb.equal(root.get("studentId"), req.getStudentId()));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        long count = examRecordDataRepo.count(specification);

        CheckExamIsStartedResp resp = new CheckExamIsStartedResp();
        resp.setIsStarted(count > 0);
        return resp;
    }

    @Override
    @ApiOperation(value = "查询考试记录-供exchange调用")
    @PostMapping("/queryExamRecordForSelectScoreByScoreId")
    public QueryExamRecordForSelectScoreResp queryExamRecordForSelectScoreByScoreId(@RequestBody QueryExamRecordForSelectScoreReq req) {
        QueryExamRecordForSelectScoreResp resp = new QueryExamRecordForSelectScoreResp();
        if (req.getExamScoreId() != null) {
            ExamScoreEntity examScore = GlobalHelper.getEntity(examScoreRepo, req.getExamScoreId(), ExamScoreEntity.class);
            ExamRecordDataEntity examRecordData =
                    GlobalHelper.getEntity(examRecordDataRepo, examScore.getExamRecordDataId(), ExamRecordDataEntity.class);
            List<ExamRecordForSelectScore> examRecordForSelectScoreList = new ArrayList<ExamRecordForSelectScore>();
            examRecordForSelectScoreList.add(buildExamRecordForSelectScore(examRecordData));
            resp.setExamRecordForSelectScoreList(examRecordForSelectScoreList);
        }
        return resp;
    }

    @Override
    @ApiOperation(value = "查询考试记录-供exchange调用")
    @PostMapping("/queryExamRecordForSelectScoreByExamStudentId")
    public QueryExamRecordForSelectScoreResp queryExamRecordForSelectScoreByExamStudentId(@RequestBody QueryExamRecordForSelectScoreReq req) {
        QueryExamRecordForSelectScoreResp resp = new QueryExamRecordForSelectScoreResp();
        if (req.getExamStudentId() != null) {
            List<ExamRecordDataEntity> examRecordList = examRecordDataRepo.findByExamStudentId(req.getExamStudentId());
            List<ExamRecordForSelectScore> examRecordForSelectScoreList = new ArrayList<ExamRecordForSelectScore>();
            for (ExamRecordDataEntity examRecordData : examRecordList) {
                examRecordForSelectScoreList.add(buildExamRecordForSelectScore(examRecordData));
            }
            resp.setExamRecordForSelectScoreList(examRecordForSelectScoreList);
        }
        return resp;
    }

    public ExamRecordForSelectScore buildExamRecordForSelectScore(ExamRecordDataEntity examRecordData) {
        ExamRecordForSelectScore examRecordForSelectScore = new ExamRecordForSelectScore();
        examRecordForSelectScore.setExamStudentId(examRecordData.getExamStudentId());
        examRecordForSelectScore.setRootOrgId(examRecordData.getRootOrgId());
        examRecordForSelectScore.setExamRecordDataId(examRecordData.getId());
        examRecordForSelectScore.setStartTime(examRecordData.getStartTime());
        examRecordForSelectScore.setEndTime(examRecordData.getEndTime());
        examRecordForSelectScore.setCleanTime(examRecordData.getCleanTime());
        examRecordForSelectScore.setIsWarn(examRecordData.getIsWarn());
        examRecordForSelectScore.setIsAudit(examRecordData.getIsAudit());
        examRecordForSelectScore.setIsIllegality(examRecordData.getIsIllegality());
        examRecordForSelectScore.setStatus(examRecordData.getExamRecordStatus().name());

        return examRecordForSelectScore;
    }

    @Override
    @ApiOperation(value = "查询照片数据-供exchange调用")
    @PostMapping("/getExamPhotoVerifyData")
    public GetExamPhotoVerifyDataResp getExamPhotoVerifyData(@RequestBody GetExamPhotoVerifyDataReq req) {
        Long scoreId = req.getScoreId();
        ExamScoreEntity examScoreEntity = GlobalHelper.getEntity(examScoreRepo, scoreId, ExamScoreEntity.class);
        if (examScoreEntity == null) {
            return null;
        }
        Long examRecordDataId = examScoreEntity.getExamRecordDataId();
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordData == null) {
            return null;
        }
        List<ExamCaptureEntity> examCaptureList = examCaptureRepo.findByExamRecordDataId(examRecordDataId);

        GetExamPhotoVerifyDataResp resp = new GetExamPhotoVerifyDataResp();
        resp.setScoreId(scoreId);
        resp.setSuccessRate(examRecordData.getFaceSuccessPercent());
        int strangerCount = 0;
        List<String> photoUrls = new ArrayList<String>();
        for (ExamCaptureEntity examCaptureEntity : examCaptureList) {
            if (examCaptureEntity.getIsStranger()) {
                strangerCount++;
            }
            photoUrls.add(examCaptureEntity.getFileUrl());
        }
        resp.setStrangerCount(strangerCount);
        resp.setPhotoUrls(photoUrls);
        return resp;
    }

    /**
     * 获取待阅卷的考试记录
     *
     * @param req
     * @return
     */
    @Override
    @ApiOperation(value = "获取待阅卷的考试记录-供bridge调用")
    @PostMapping("/getToBeMarkExamRecord")
    public GetToBeMarkExamRecordResp getToBeMarkExamRecord(@RequestBody GetToBeMarkExamRecordReq req) {
        if (null == req.getExamStudentIdList() || req.getExamStudentIdList().isEmpty()) {
            throw new StatusException("100101", "考生id不允许为空");
        }

        GetToBeMarkExamRecordResp resp = new GetToBeMarkExamRecordResp();
        //待阅卷的考试记录集合
        List<ToBeMarkExamRecordBean> toBeMarkExamRecordBeanList = new ArrayList<>();
        for (Long examStudentId : req.getExamStudentIdList()) {
            ExamStudentEntity examStudent = examStudentRepo.findByExamStudentId(examStudentId);

            if (null == examStudent.getStudentId()) {
                throw new StatusException("100101", "考生id：" + examStudentId + "不正确");
            }

            Long examId = examStudent.getExamId();
            //待阅的考试列表
            List<ExamRecordForMarkingEntity> examRecordForMarkingList =
                    examRecordForMarkingService.queryValidExamRecordList(examId, examStudent.getCourseId());

            //未参加考试或未违纪的考试数据略过
            if (null == examRecordForMarkingList || examRecordForMarkingList.isEmpty()) {
                continue;
            }

            for (ExamRecordForMarkingEntity markingRecord : examRecordForMarkingList) {
                Long examRecordDataId = markingRecord.getExamRecordDataId();
                ExamRecordDataEntity examRecordData =
                        GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);

                ToBeMarkExamRecordBean toBeMarkExamRecordBean = new ToBeMarkExamRecordBean();
                toBeMarkExamRecordBean.setExamId(markingRecord.getExamId());
                toBeMarkExamRecordBean.setStudentName(examRecordData.getStudentName());
                toBeMarkExamRecordBean.setStudentCode(examRecordData.getStudentCode());

                CourseCacheBean course = CacheHelper.getCourse(examRecordData.getCourseId());
                toBeMarkExamRecordBean.setCourseCode(course.getCode());
                toBeMarkExamRecordBean.setCourseName(course.getName());

                toBeMarkExamRecordBean.setPaperType(markingRecord.getPaperType());
                toBeMarkExamRecordBean.setExamStudentId(markingRecord.getExamStudentId());
                toBeMarkExamRecordBean.setExamRecordDataId(markingRecord.getExamRecordDataId());
                //待评阅的主观题集合
                toBeMarkExamRecordBean.setSubjectiveAnswerList(
                        getToBeMarkSubjectiveAnswerList(examRecordData.getExamRecordQuestionsId(),
                                examRecordDataId, examId, course.getCode(), examRecordData.getPaperType()));

                toBeMarkExamRecordBeanList.add(toBeMarkExamRecordBean);
            }

        }
        resp.setToBeMarkExamRecordBeanList(toBeMarkExamRecordBeanList);
        return resp;
    }

    /**
     * 获取部分考试记录数据
     *
     * @param req
     * @return
     */
    @Override
    @ApiOperation(value = "获取部分考试记录数据")
    @PostMapping("/getPartialExamRecordData")
    public GetPartialExamRecordDataResp getPartialExamRecordData(@RequestBody GetPartialExamRecordDataReq req) {
        ExamRecordDataEntity examRecordData =
                GlobalHelper.getEntity(examRecordDataRepo, req.getExamRecordDataId(), ExamRecordDataEntity.class);

        if (null == examRecordData) {
            return new GetPartialExamRecordDataResp();
        }

        GetPartialExamRecordDataResp resp = new GetPartialExamRecordDataResp();
        resp.setExamStudentId(examRecordData.getExamStudentId());

        return resp;
    }

    /**
     * 获取待阅卷的考试记录
     *
     * @param req
     * @return
     */
    @Override
    @ApiOperation(value = "分页获取待阅卷的考试记录")
    @PostMapping("/getPagedToBeMarkExamRecord")
    public GetPagedToBeMarkExamRecordResp getPagedToBeMarkExamRecord(@RequestBody GetPagedToBeMarkExamRecordReq req) {
        Long st = System.currentTimeMillis();

        Long examId = req.getExamId();
        String courseCode = req.getSubjectCode();
        Long startId = req.getStartId();
        Integer size = req.getSize();

        validateToBeMarkData(examId, courseCode, startId, size);

        Long startTime = System.currentTimeMillis();

        List<ExamStudentEntity> limitedExamStuList =
                examStudentRepo.getLimitExamStudentList(examId, courseCode, startId, size);

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("1.[GET_PAGED_TO_BE_MARK_EXAM_RECORD-" + examId + "-" + courseCode + "-" + startId + "]" +
                    "获取考生耗时:" + (System.currentTimeMillis() - startTime) + " ms");
        }

        startTime = System.currentTimeMillis();

        GetPagedToBeMarkExamRecordResp resp = new GetPagedToBeMarkExamRecordResp();
        Long nextId = startId;

        if (null == limitedExamStuList || limitedExamStuList.isEmpty()) {
            resp.setNextId(nextId);
            resp.setToBeMarkExamRecordBeanList(null);

            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("999.[GET_PAGED_TO_BE_MARK_EXAM_RECORD-" + examId + "-" + courseCode + "-" + startId + "]end..." +
                        "未找到对应的考生数据，合计耗时:" + (System.currentTimeMillis() - st) + " ms");
            }

            return resp;
        }

        List<PagedToBeMarkExamRecordBean> pagedToBeMarkList = new ArrayList<>();

        CourseCacheBean course = CacheHelper.getCourse(limitedExamStuList.get(0).getCourseId());

        int si = 0;//考生索引
        for (ExamStudentEntity examStu : limitedExamStuList) {
            si++;

            Long st1 = System.currentTimeMillis();

            //当前考生待阅卷的考试记录
            List<ExamRecordForMarkingEntity> examRecordForMarkingList =
                    examRecordForMarkingService.queryValidExamRecordList(examStu.getExamStudentId());

            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("1-2." + si + "[GET_PAGED_TO_BE_MARK_EXAM_RECORD-" + examId + "-" + courseCode + "-" + examStu.getExamStudentId() + "]" +
                        "获取考生待阅卷的考试记录耗时:" + (System.currentTimeMillis() - st1) + " ms");
            }

            //如果考生找不到待阅卷的数据，直接跳过
            if (null == examRecordForMarkingList || examRecordForMarkingList.isEmpty()) {
                continue;
            }

            st1 = System.currentTimeMillis();

            for (ExamRecordForMarkingEntity record : examRecordForMarkingList) {
                PagedToBeMarkExamRecordBean pagedBean = new PagedToBeMarkExamRecordBean();
                pagedBean.setExamId(examId);
                pagedBean.setStudentName(examStu.getStudentName());
                pagedBean.setStudentCode(examStu.getStudentCode());
                pagedBean.setIdentityNumber(examStu.getIdentityNumber());
                pagedBean.setCourseCode(courseCode);
                pagedBean.setCourseName(course.getName());
                pagedBean.setPaperType(examStu.getPaperType());
                pagedBean.setExamStudentId(examStu.getExamStudentId());
                pagedBean.setExamRecordDataId(record.getExamRecordDataId());
                pagedBean.setGrade(examStu.getGrade());

                pagedBean.setSubjectiveAnswerList(getSubjectiveAnswerList(record.getExamRecordDataId()));

                pagedToBeMarkList.add(pagedBean);
            }

            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("1-2." + si + "[GET_PAGED_TO_BE_MARK_EXAM_RECORD-" + examId + "-" + courseCode + "-" + examStu.getExamStudentId() + "]" +
                        "构建带作答记录的待阅卷的考试记录耗时:" + (System.currentTimeMillis() - st1) + " ms");
            }
        }

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("2.[GET_PAGED_TO_BE_MARK_EXAM_RECORD-" + examId + "-" + courseCode + "-" + startId + "]" +
                    "获取" + size + "条考生的待阅卷记录共耗时:" + (System.currentTimeMillis() - startTime) + " ms");
        }

        nextId = limitedExamStuList.get(limitedExamStuList.size() - 1).getExamStudentId() + 1;
        resp.setNextId(nextId);
        resp.setToBeMarkExamRecordBeanList(pagedToBeMarkList);

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("999.[GET_PAGED_TO_BE_MARK_EXAM_RECORD-" + examId + "-" + courseCode + "-" + startId + "]end..." +
                    "合计耗时:" + (System.currentTimeMillis() - st) + " ms");
        }
        return resp;
    }

    /**
     * 获取主观题集合
     *
     * @param examRecordDataId
     * @return
     */
    private List<PagedToBeMarkSubjectiveAnswerBean> getSubjectiveAnswerList(Long examRecordDataId) {
        //主观题作答集合
        List<ExamQuestionEntity> eqList = examRecordQuestionsService.querySubjectiveAnswerList(examRecordDataId);

        List<PagedToBeMarkSubjectiveAnswerBean> resultList = new ArrayList<>();
        for (ExamQuestionEntity eq : eqList) {
            PagedToBeMarkSubjectiveAnswerBean bean = new PagedToBeMarkSubjectiveAnswerBean();
            bean.setMainNumber(eq.getMainNumber());
            bean.setOrder(eq.getOrder());
            bean.setQuestionId(eq.getQuestionId());
            bean.setStudentAnswer(eq.getStudentAnswer());
            bean.setAnswerType(eq.getAnswerType());
            bean.setQuestionType(eq.getQuestionType());

            resultList.add(bean);
        }

        return resultList;
    }

    /**
     * 校验待阅卷数据
     *
     * @param examId
     * @param courseCode
     * @param startId
     * @param size
     */
    private void validateToBeMarkData(Long examId, String courseCode, Long startId, Integer size) {
        if (null == examId) {
            throw new StatusException("101001", "考生id不允许为空");
        }

        if (StringUtils.isNullOrEmpty(courseCode)) {
            throw new StatusException("101002", "科目代码不允许为空");
        }

        if (null == startId) {
            throw new StatusException("101003", "考试记录id不允许为空");
        }

        if (null == size) {
            throw new StatusException("101004", "数据量大小不允许为空");
        }

        if (size.intValue() > 500) {
            throw new StatusException("101005", "数据量最大不得超过500");
        }
    }

    /**
     * 获取待评阅的主观题集合
     *
     * @param recordQuestionsId
     * @param examRecordDataId
     * @param examId
     * @param courseCode
     * @param paperType
     * @return
     */
    private List<ToBeMarkSubjectiveAnswerBean> getToBeMarkSubjectiveAnswerList(String recordQuestionsId,
                                                                               Long examRecordDataId, Long examId, String courseCode, String paperType) {
        ExamRecordQuestionsEntity questionsEntity;
        if (null != recordQuestionsId) {
            questionsEntity = GlobalHelper.getEntity(examRecordQuestionsRepo, recordQuestionsId, ExamRecordQuestionsEntity.class);
        } else {
            questionsEntity = examRecordQuestionsService.getExamRecordQuestionsAndFixExamRecordDataIfNecessary(examRecordDataId);
        }

        List<ExamQuestionEntity> examQuestionList = questionsEntity.getExamQuestionEntities();
        //获取所有的主观题
        List<ExamQuestionEntity> subjectiveQuestionList = examQuestionList.stream().filter(p -> p.getQuestionType() == QuestionType.ESSAY ||
                p.getQuestionType() == QuestionType.FILL_UP).collect(Collectors.toList());

        List<ToBeMarkSubjectiveAnswerBean> subjectiveAnswerBeanList = new ArrayList<>();

        for (int i = 0; i < subjectiveQuestionList.size(); i++) {
            ExamQuestionEntity sq = subjectiveQuestionList.get(i);

            ToBeMarkSubjectiveAnswerBean subjectiveAnswerBean;

            //集合中是否已存在当前作答结果（因为套题会提前插入部分数据）
            String questionId = sq.getQuestionId();
            Integer order = sq.getOrder();
            List<ToBeMarkSubjectiveAnswerBean> existSubjectiveAnswerBeanList = subjectiveAnswerBeanList.stream().filter(p -> questionId.equals(p.getQuestionId())
                    && order.equals(p.getOrder())).collect(Collectors.toList());

            //如果已经存在，则直接修改,因为套题一次性设置多个小题的题干，所以如果已经设置过就不再设置题干部分
            if (null != existSubjectiveAnswerBeanList && !existSubjectiveAnswerBeanList.isEmpty()) {
                subjectiveAnswerBean = existSubjectiveAnswerBeanList.get(0);

                subjectiveAnswerBean.setMainNumber(sq.getMainNumber());
                subjectiveAnswerBean.setOrder(sq.getOrder());
                subjectiveAnswerBean.setQuestionId(sq.getQuestionId());
                subjectiveAnswerBean.setStudentAnswer(transformedStudentAnswer(sq.getStudentAnswer()));//格式化过的答案
                subjectiveAnswerBean.setAnswerType(sq.getAnswerType());
                subjectiveAnswerBean.setRealAnswerType(transformedAnswerType(sq.getAnswerType(), sq.getQuestionType(), examId));//实际的作答类型

                subjectiveAnswerBean.setAnswer(sq.getCorrectAnswer());
            }

            //不存在则重新实例化,并添加
            else {
                subjectiveAnswerBean = new ToBeMarkSubjectiveAnswerBean();

                subjectiveAnswerBean.setMainNumber(sq.getMainNumber());
                subjectiveAnswerBean.setOrder(sq.getOrder());
                subjectiveAnswerBean.setQuestionId(sq.getQuestionId());
                subjectiveAnswerBean.setStudentAnswer(transformedStudentAnswer(sq.getStudentAnswer()));//格式化过的答案
                subjectiveAnswerBean.setAnswerType(sq.getAnswerType());
                subjectiveAnswerBean.setRealAnswerType(transformedAnswerType(sq.getAnswerType(), sq.getQuestionType(), examId));//实际的作答类型

                subjectiveAnswerBean.setAnswer(sq.getCorrectAnswer());

                /**题干部分相关处理逻辑--start**/
                //TODO 这是由于历史原因(套题下的小题共享同一个questionId)，导致这个恶心的变通实现方式,如果以后优化了套题的结构，可修改
                QuestionCacheBean getQuestionResp = CacheHelper.getQuestion(examId,
                        courseCode, paperType, questionId);//获取题干相关信息
                DefaultQuestionStructure questionStructure = getQuestionResp.getDefaultQuestion().getMasterVersion();

                //所有的小题单元
                List<DefaultQuestionUnit> questionUnits = questionStructure.getQuestionUnitList();

                //如果主题干不为空，则认为是套题(套题需要将大题题干和小题题干拼在一起)
                String mainBody = questionStructure.getBody();//主题干
                if (!StringUtils.isNullOrEmpty(mainBody)) {

                    //当前order对应的小题在此套题中的索引
                    int currentIndex = 0;
                    for (int j = 0; j < questionUnits.size(); j++) {

                        //判断当前套题中的小题是否为主观题，只有主观题才需要添加进来
                        if (!isSubjectiveQuestion(questionUnits.get(j).getQuestionType())) {
                            continue;
                        }

                        //拼装第一条套题下的小题，（本条数据是完整的）,相当于全部添加，无需更新
                        if (StringUtils.isNullOrEmpty(subjectiveAnswerBean.getBody())) {
                            currentIndex = j;
                            //第1个子主观题的题干（不一定是套题的第一题）
                            String subBody0 = questionUnits.get(j).getBody();

                            subjectiveAnswerBean.setBody(mainBody + subBody0);//构建题干
                            subjectiveAnswerBeanList.add(subjectiveAnswerBean);
                        }
                        //拼装套题的其它几个小题的题干（只初始化题干和唯一标识）,并提前添加到结果集中
                        else {
                            ToBeMarkSubjectiveAnswerBean nextSubjectiveAnswerBean = new ToBeMarkSubjectiveAnswerBean();
                            nextSubjectiveAnswerBean.setQuestionId(questionId);
                            nextSubjectiveAnswerBean.setOrder(order + (j - currentIndex));
                            nextSubjectiveAnswerBean.setBody(mainBody + questionUnits.get(j).getBody());//构建题干
                            subjectiveAnswerBeanList.add(nextSubjectiveAnswerBean);
                        }

                    }
                }
                //不是套题，则默认取第一条记录的题干
                else {
                    subjectiveAnswerBean.setBody(questionUnits.get(0).getBody());//构建题干
                    subjectiveAnswerBeanList.add(subjectiveAnswerBean);
                }
                /**题干部分相关处理逻辑--end**/

            }
        }

        return subjectiveAnswerBeanList;
    }

    /**
     * 当前题型是否为主观题
     *
     * @param questionType
     * @return
     */
    private boolean isSubjectiveQuestion(QuestionType questionType) {
        return (QuestionType.ESSAY == questionType || QuestionType.FILL_UP == questionType);
    }

    /**
     * 根据原始作答类型和学生实际作答推断出真实的作答类型
     *
     * @param answerType   原始作答类型
     * @param questionType 题型
     * @param examId       考试id
     * @return
     */
    private String transformedAnswerType(AnswerType answerType, QuestionType questionType, Long examId) {
        //如果题型为问答题，且作答类型不为音频作答，且开启微信作答，则此题为图片作答题
        if (questionType == QuestionType.ESSAY &&
                (null == answerType || (null != answerType && answerType != AnswerType.SINGLE_AUDIO))) {
            if (ExamCacheTransferHelper.weixinAnswerEnabled(examId)) {
                return "image";
            }
        }

        if (null != answerType && answerType == AnswerType.SINGLE_AUDIO) {
            return "audio";
        }

        return "text";
    }

    /**
     * 转化过的作答
     *
     * @param studentAnswer
     * @return
     */
    private String transformedStudentAnswer(String studentAnswer) {
        if (StringUtils.isNullOrEmpty(studentAnswer)) {
            return studentAnswer;
        }

        //图片题特殊处理(因为图片作答题中有html标签)
        Document doc = Jsoup.parse(studentAnswer);
        Elements imgElements = doc.select("img[src]");
        String imgStudentAnswer = "";
        for (Element el : imgElements) {
            String src = el.attr("src");
            if (!StringUtils.isNullOrEmpty(src)) {
                imgStudentAnswer += src + "|";
            }
        }
        if (!StringUtils.isNullOrEmpty(imgStudentAnswer)) {
            return imgStudentAnswer.substring(0, imgStudentAnswer.length() - 1);
        }

        return studentAnswer;
    }


    @Override
    @ApiOperation(value = "获取考试监考待审记录数")
    @PostMapping("/getGetAwaitingAuditCount")
    public GetAwaitingAuditCountResp getGetAwaitingAuditCount(@RequestBody GetAwaitingAuditCountReq req) {
        if (req.getRootOrgId() == null) {
            throw new StatusException("500", "RootOrgId不允许为空");
        }
        if (req.getExamId() == null) {
            throw new StatusException("500", "ExamId不允许为空");
        }
        Long count = examRecordDataService.getGetAwaitingAuditCount(req.getRootOrgId(), req.getExamId());
        GetAwaitingAuditCountResp res = new GetAwaitingAuditCountResp();
        res.setCount(count);
        return res;
    }

	@Override
	@ApiOperation(value = "检查试卷是否已使用")
    @PostMapping("/checkPaperInExam")
	public CheckPaperInExamResp checkPaperInExam(@RequestBody CheckPaperInExamReq req) {
		if (org.apache.commons.lang3.StringUtils.isBlank(req.getBasePaperId())) {
            throw new StatusException("500", "basePaperId不允许为空");
        }
		return examRecordDataService.checkPaperInExam(req);
	}

}
