/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-15 14:15:05.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.OeExamStudentCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamStudentCountBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.FindStudentType;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.OeExamStudentBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ToBeMarkExamStudentBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.*;
import cn.com.qmth.examcloud.core.oe.admin.api.response.*;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordForMarkingRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordForMarkingEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 考生信息接口
 *
 * @author: QMTH
 * @since: 2018/8/15
 */
@Api(tags = "考生信息相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/exam/student")
public class ExamStudentCloudServiceProvider extends ControllerSupport implements OeExamStudentCloudService {
    /**
     *
     */
    private static final long serialVersionUID = 1638873981075522570L;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamRecordForMarkingRepo examRecordForMarkingRepo;

    @Deprecated
    @ApiOperation(value = "同步考生(全部)数据")
    @PostMapping("/sync/all/data")
    public ExamStudentSyncAllDataResp syncExamStudentAllData(@RequestBody ExamStudentSyncAllDataReq req) {
        /*List<ExamStudentInfo> examStudents = ExamStudentBeanConvert.of(req);
        examStudentService.syncExamStudentAllData(examStudents);
        return new ExamStudentSyncAllDataResp();*/
        return null;
    }

    @Deprecated
    @ApiOperation(value = "同步考生(部分)数据")
    @PostMapping("/sync/part/data")
    public ExamStudentSyncPartDataResp syncExamStudentPartData(@RequestBody ExamStudentSyncPartDataReq req) {
        /*List<ExamStudentPartInfo> examStudents = ExamStudentBeanConvert.of(req);
        examStudentService.syncExamStudentPartData(examStudents);
        return new ExamStudentSyncPartDataResp();*/
        return null;
    }

    @Override
    @ApiOperation(value = "根据条件查询考生")
    @PostMapping("/findExamStudentInfoBy")
    public FindExamStudentInfoResp findExamStudentInfoBy(@RequestBody FindExamStudentInfoReq req) {
        FindStudentType findStudentType = req.getFindStudentType();
        if (findStudentType == null) {
            throw new StatusException("ExamStudentCloudServiceProvider-findExamStudentInfoBy-exception", "查询方式不能为空");
        }
        FindExamStudentInfoResp resp = new FindExamStudentInfoResp();

        if (findStudentType == FindStudentType.BY_EXAMID) {
            Long examId = req.getFindStudentBean().getExamId();
            if (examId != null) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByExamId(examId);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        } else if (findStudentType == FindStudentType.BY_EXAMID_AND_IDENTITYNUMBERS) {
            Long examId = req.getFindStudentBean().getExamId();
            List<String> identityNumbes = req.getFindStudentBean().getIdentityNumbers();
            if (examId != null && identityNumbes != null && identityNumbes.size() > 0) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByExamIdAndIdentityNumberIn(examId, identityNumbes);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        } else if (findStudentType == FindStudentType.BY_IDENTITYNUMBERS_AND_ROOTORGID) {
            Long rootOrgId = req.getFindStudentBean().getRootOrgId();
            List<String> identityNumbes = req.getFindStudentBean().getIdentityNumbers();
            if (rootOrgId != null && identityNumbes != null && identityNumbes.size() > 0) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByRootOrgIdAndIdentityNumberIn(rootOrgId, identityNumbes);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        } else if (findStudentType == FindStudentType.BY_EXAMID_AND_STUDENTCODES) {
            Long examId = req.getFindStudentBean().getExamId();
            List<String> studentCodes = req.getFindStudentBean().getStudentCodes();
            if (examId != null && studentCodes != null && studentCodes.size() > 0) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByExamIdAndStudentCodeIn(examId, studentCodes);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        }
        return resp;
    }

    @Override
    @ApiOperation(value = "解绑考生学号")
    @PostMapping("/unbindExamStudent")
    public UnbindExamStudentResp unbindExamStudent(UnbindExamStudentReq req) {
        UnbindExamStudentResp resp = new UnbindExamStudentResp();
        if (StringUtils.isNullOrEmpty(req.getIdentityNumber())) {
            resp.setSuccess(false);
            resp.setMsg("身份证号不允许为空");
            return resp;
        }
        if (null == req.getStudentCodeList() || req.getStudentCodeList().isEmpty()) {
            resp.setSuccess(false);
            resp.setMsg("学号不允许为空");
            return resp;
        }
        examStudentRepo.updateExamStudentCode(req.getIdentityNumber(), req.getStudentCodeList());
        resp.setSuccess(true);
        return resp;
    }

    @Override
    @ApiOperation(value = "根据考试批次等查询条件获取考生数量(一次最多返回100条数据)")
    @PostMapping("/getExamStudentCount")
    public GetExamStudentCountResp getExamStudentCount(@RequestBody GetExamStudentCountReq req) {
        if (req.getExamIdList() == null || req.getExamIdList().size() == 0) {
            throw new StatusException("100001", "考试id不允许为空");
        }

        if (req.getExamIdList().size() > 100) {
            throw new StatusException("100002", "考试id个数不得超过100个");
        }

        Long count;
        List<ExamStudentCountBean> countBeanList = new ArrayList<>();
        for (Long examId : req.getExamIdList()) {
            if (req.getFinishedExam() == null) {
                count = examStudentRepo.countByExamId(examId);
            } else {
                count = examStudentRepo.countByExamIdAndFinished(examId, req.getFinishedExam());
            }

            ExamStudentCountBean countBean = new ExamStudentCountBean();
            countBean.setExamId(examId);
            countBean.setCount(count == null ? 0 : count);
            countBeanList.add(countBean);
        }

        GetExamStudentCountResp resp = new GetExamStudentCountResp();
        resp.setCountList(countBeanList);
        return resp;
    }

    /**
     * 根据查询条件获取所有待阅卷的考生
     *
     * @param req
     * @return
     */
    @Override
    @ApiOperation(value = "获取待阅卷的考生记录")
    @PostMapping("/getToBeMarkExamStudent")
    public GetToBeMarkExamStudentResp getToBeMarkExamStudent(@RequestBody GetToBeMarkExamStudentReq req) {
        List<ExamRecordForMarkingEntity> examRecordForMarkingList = examRecordForMarkingRepo.findByExamId(req.getExamId());
        GetToBeMarkExamStudentResp resp = new GetToBeMarkExamStudentResp();

        List<Long> examStudentIdList = examRecordForMarkingList.stream().
                map(p -> p.getExamStudentId()).distinct().collect(Collectors.toList());

        List<ToBeMarkExamStudentBean> examStudentBeanList = new ArrayList<>();
        for (Long examStudentId : examStudentIdList) {
            ToBeMarkExamStudentBean examStudentBean = new ToBeMarkExamStudentBean();
            examStudentBean.setExamStudentId(examStudentId);
            examStudentBeanList.add(examStudentBean);
        }

        resp.setExamStudentList(examStudentBeanList);
        return resp;
    }

    private List<OeExamStudentBean> buildExamStudentBeans(List<ExamStudentEntity> examStudents) {
        List<OeExamStudentBean> examStudentBeans = new ArrayList<OeExamStudentBean>();
        for (ExamStudentEntity examStudent : examStudents) {
            OeExamStudentBean examStudentBean = new OeExamStudentBean();
            examStudentBean.setExamStudentId(examStudent.getExamStudentId());
            examStudentBean.setExamId(examStudent.getExamId());
            examStudentBean.setStudentCode(examStudent.getStudentCode());
            examStudentBean.setStudentName(examStudent.getStudentName());
            examStudentBean.setIdentityNumber(examStudent.getIdentityNumber());
            examStudentBean.setCourseCode(examStudent.getCourseCode());
            examStudentBean.setFinished(examStudent.getFinished());
            examStudentBeans.add(examStudentBean);
        }
        return examStudentBeans;
    }
}