package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import java.util.ArrayList;
import java.util.DoubleSummaryStatistics;
import java.util.List;
import java.util.stream.Collectors;

import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamStudentDataCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamStudentDataBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamStudentScoreDataBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetExamStudentDataReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetExamStudentDataResp;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordQuestionsRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.admin.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.ExamStudentEffectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @Description 获取考生考试相关数据接口
 * @Author lideyin
 * @Date 2019/7/18 11:10
 * @Version 1.0
 */
@Api(tags = "获取考生考试相关数据接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/examStudentData")
public class ExamStudentDataCloudServiceProvider extends ControllerSupport implements ExamStudentDataCloudService {
    /**
     *
     */
    private static final long serialVersionUID = 4538819077863159166L;
    @Autowired
    ExamRecordQuestionsRepo examRecordQuestionsRepo;
    @Autowired
    private ExamStudentService examStudentService;
    @Autowired
    private GainBaseDataService gainBaseDataService;
    @Autowired
    private ExamRecordService examRecordService;

    @Override
    @ApiOperation(value = "获取考生分数相关数据")
    @PostMapping("/getExamStudentData")
    public GetExamStudentDataResp getExamStudentData(@RequestBody GetExamStudentDataReq req) {
        validateRequestData(req);
        Long nextId = req.getStartId();

        GetExamStudentDataResp resp = new GetExamStudentDataResp();
        List<ExamStudentInfo> examStudentList = examStudentService.getLimitExamStudentList(req.getExamId(), req.getStartId(), req.getSize());
        if (examStudentList == null || examStudentList.isEmpty()) {
            resp.setNextId(nextId);
            return resp;
        }

        nextId = examStudentList.get(examStudentList.size() - 1).getId();
        nextId++;
        resp.setNextId(nextId);

        List<Long> examStudentIdList = examStudentList.stream().map(p -> p.getExamStudentId()).collect(Collectors.toList());
        //阅卷方式
        String markingType = ExamCacheTransferHelper.
                getDefaultCachedExamProperty(req.getExamId(), ExamProperties.MARKING_TYPE.name()).getValue();
        List<ExamStudentEffectiveScoreInfo> effectiveScoreList = examRecordService.getExamStudentEffectiveScoreList(markingType, examStudentIdList);
        List<ExamRecordQuestionsEntity> questionList = null;
        if (effectiveScoreList != null && !effectiveScoreList.isEmpty()) {
            List<Long> examRecordDataIdList = effectiveScoreList.stream().map(p -> p.getExamRecordDataId()).collect(Collectors.toList());
            questionList = examRecordQuestionsRepo.findByExamRecordDataIdIn(examRecordDataIdList);
            //错误数据的特殊处理
            String strErrorExamRecordDataIds = "";
            List<Long> errorExamRecordDataIdList;
            if (questionList == null || questionList.isEmpty()) {
                errorExamRecordDataIdList = effectiveScoreList.stream().map(p -> p.getExamRecordDataId()).collect(Collectors.toList());
                for (Long eid : errorExamRecordDataIdList) {
                    strErrorExamRecordDataIds += eid + ",";
                }
                LOGGER.error("[GETEXAMSTUDENTDATA],200105考试作答数据异常,错误的考试记录id:" + strErrorExamRecordDataIds);
                throw new StatusException("200105", "考试作答数据异常,相关考试记录id为：" + strErrorExamRecordDataIds);
            }
            if (questionList.size() != effectiveScoreList.size()) {
                List<Long> finalExamRecordDataIdList = questionList.stream().map(p -> p.getExamRecordDataId()).collect(Collectors.toList());
                errorExamRecordDataIdList = effectiveScoreList.stream().filter(p -> !finalExamRecordDataIdList.contains(p.getExamRecordDataId())).map(p -> p.getExamRecordDataId()).collect(Collectors.toList());
                for (Long eid : errorExamRecordDataIdList) {
                    strErrorExamRecordDataIds += eid + ",";
                }
                LOGGER.error("[GETEXAMSTUDENTDATA],200106考试作答数据异常,错误的考试记录id:" + strErrorExamRecordDataIds);
                throw new StatusException("200106", "考试作答数据异常,相关考试记录id为：" + strErrorExamRecordDataIds);
            }
        }
        List<ExamStudentDataBean> examStudentDataBeanList = new ArrayList<>();
        for (ExamStudentInfo es : examStudentList) {
            ExamStudentDataBean esBean = new ExamStudentDataBean();
            ExamStudentEffectiveScoreInfo score = null;

            esBean.setExamStudentId(es.getExamStudentId());
            esBean.setExamId(es.getExamId());
            esBean.setCourseCode(es.getCourseCode());
            esBean.setCourseId(es.getCourseId());
            esBean.setAbsent(!es.getFinished());
            esBean.setRootOrgId(es.getRootOrgId());
            esBean.setOrgId(es.getOrgId());

            if (effectiveScoreList != null && !effectiveScoreList.isEmpty()) {
                List<ExamStudentEffectiveScoreInfo> scoreList = effectiveScoreList.stream().
                        filter(p -> p.getExamStudentId().equals(es.getExamStudentId())).collect(Collectors.toList());
                if (scoreList != null && !scoreList.isEmpty()) {
                    score = scoreList.get(0);
                }
                if (score != null) {
                    esBean.setBasePaperId(score.getBasePaperId());
                    //考生总得分
                    esBean.setScore(score.getScore());

                    ExamStudentEffectiveScoreInfo finalScore = score;
                    //试卷总分
                    List<ExamRecordQuestionsEntity> curQuestionList = questionList.stream().
                            filter(p -> p.getExamRecordDataId().equals(finalScore.getExamRecordDataId())).collect(Collectors.toList());
//                    if (curQuestionList==null || curQuestionList.isEmpty()){
//                        //正常的数据不会出现这种情况con,暂时先记日志
//                        log.error("[GETEXAMSTUDENTDATA]200106考试作答记录数据异常,examRecordDataId:"+finalScore.getExamRecordDataId());
////                        throw new StatusException("200106","考试作答记录数据异常");
//                        continue;
//                    }
                    ExamRecordQuestionsEntity examRecordQuestionsEntity = curQuestionList.get(0);
                    List<ExamQuestionEntity> examQuestionList = examRecordQuestionsEntity.getExamQuestionEntities();
                    DoubleSummaryStatistics questionScoreStatistics = examQuestionList.stream().
                            collect(Collectors.summarizingDouble(ExamQuestionEntity::getQuestionScore));

                    esBean.setTotalScore(questionScoreStatistics.getSum());
                    List<ExamStudentScoreDataBean> examStudentScoreDetailList = new ArrayList<>();
                    for (ExamQuestionEntity eq : examQuestionList) {
                        ExamStudentScoreDataBean essBean = new ExamStudentScoreDataBean();
                        essBean.setQuestionId(eq.getQuestionId());
                        essBean.setQuestionOrder(eq.getOrder());
                        essBean.setScore(eq.getStudentScore());
                        essBean.setTotalScore(eq.getQuestionScore());
                        examStudentScoreDetailList.add(essBean);
                    }
                    esBean.setScoreDetails(examStudentScoreDetailList);
                }
            }
            examStudentDataBeanList.add(esBean);
        }
        resp.setList(examStudentDataBeanList);

        return resp;
    }

    private void validateRequestData(GetExamStudentDataReq req) {
        if (req.getStartId() == null) {
            throw new StatusException("200101", "起始考生id不允许为空");
        }
        if (req.getExamId() == null) {
            throw new StatusException("200102", "考试id不允许为空");
        }
        if (req.getSize() == null) {
            throw new StatusException("200103", "数据条数不允许为空");
        }
        Integer maxSize = PropertyHolder.getInt("exam.oe.getExamStudentData.maxSize", 200);
        if (req.getSize() > maxSize) {
            throw new StatusException("200104", "该接口一次最多只能返回" + maxSize + "条数据");
        }
    }
}
