/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-23 16:34:26.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service;

import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.ExamStudentEffectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamStudentQuestionScoreInfo;

import org.springframework.data.domain.Page;

import java.util.List;

/**
 * 考试记录接口
 *
 * @author: QMTH
 * @since: 2018/8/15
 */
public interface ExamRecordService {

    /**
     * 查询考试记录列表(分页)
     *
     * @param query
     * @return
     */
    Page<ExamRecordInfo> getExamRecordList(ExamRecordQuery query);

    /**
     * 查询“考试明细”列表(分页)
     *
     * @param query
     * @return
     */
    Page<ExamRecordInfo> getExamRecordDetailListForPage(ExamRecordQuery query);

    /**
     * 查询“监考待审”列表(分页)
     *
     * @param query
     * @return
     */
    Page<ExamRecordInfo> getExamRecordWaitingAuditList(ExamRecordQuery query);

    /**
     * 查询“监考待审”列表(分页)
     *
     * @param query
     * @return
     */
    List<ExamRecordInfo> getExamRecordWaitingAudit(ExamRecordQuery query);

	/**
	 * Description 查询“监考待审”下一条记录
	 *
	 * @param query
	 * @param examRecordDataId
	 * @param next
	 * @return java.lang.Long
	 */
	Long getExamRecordWaitingAuditNextId(ExamRecordQuery query, Long examRecordDataId, String next);

    /**
     * 获取正在考试的考生ID列表
     *
     * @param examId
     * @return
     */
    List<Long> getExamRecordingStudentIds(Long examId);

    /**
     * 获取某个考生的考试记录列表
     *
     * @param examStudentId
     * @return
     */
    List<ExamRecordDataEntity> getExamRecordListByExamStudentId(Long examStudentId);

    /**
     * 查询“考试明细” 是否存在待审数据
     * @param query
     * @return
     */
	Long existsWarnExamRecordDetail(ExamRecordQuery query);
	
	/**
	 * 根据条件查询 “考试明细”
	 * @param newQuery
	 * @return
	 */
	List<ExamRecordInfo> getExamRecordDetailList(ExamRecordQuery newQuery);

	/**
	 * 根据考试id和课程代码获取学生作答成绩
	 * @param examId
	 * @param courseCode
	 * @return
	 */
	List<ExamStudentQuestionScoreInfo> getExamStudentQuestionScoreList(Long examId, String courseCode);

    /**
     * 获取考生的最终有效成绩
     * @param markingType
     * @param examStudentIdList
     * @return
     */
    List<ExamStudentEffectiveScoreInfo> getExamStudentEffectiveScoreList(String markingType, List<Long> examStudentIdList);

	List<ExamRecordInfo> getExamRecordDetailListForAsync(ExamRecordQuery query);

	void refreshCaptureStatistic(Long examId);
}