package cn.com.qmth.examcloud.core.oe.admin.service.bean.exporttask;

import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExportTaskEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExportTaskType;
import cn.com.qmth.examcloud.core.oe.admin.service.AsyncExportService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExportTaskService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examaudit.ExamAuditQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentQuery;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExportTask extends Thread {

	private static final Logger log = LoggerFactory.getLogger(ExportTask.class);

	private static int cacheLockTimeout = 60 * 2;

	private AsyncExportService asyncExportService = SpringContextHolder.getBean(AsyncExportService.class);

	private ExportTaskService exportTaskService = SpringContextHolder.getBean(ExportTaskService.class);

	private RedisClient redisClient = SpringContextHolder.getBean(RedisClient.class);

	@Override
	public void run() {
		for (;;) {
			try {
				log.info("ExportTask start");
				ExportTaskEntity et = exportTaskService.findExportTaskToDispose();
				if (et == null) {
					log.info("ExportTask sleep 30s");
					Util.sleep(30);
					continue;
				}

				String cacheLock = "$_CACHE_LOCK:OE_EXPORT_TASK:" + et.getId();
				Boolean lock = redisClient.setIfAbsent(cacheLock, cacheLock, cacheLockTimeout);
				if (!lock) {
					log.info("ExportTask lock... " + cacheLock);
					continue;
				}

				try {
					exportTaskService.startExportTask(et.getId());
					if (ExportTaskType.EXAM_DETAIL.equals(et.getType())) {
						examRecordDetails(et);
					} else if (ExportTaskType.SCORE_STATISTIC.equals(et.getType())) {
						examScores(et);
					} else if (ExportTaskType.EXAM_SCHEDULING.equals(et.getType())) {
						examScheduling(et);
					} else if (ExportTaskType.AUDIT.equals(et.getType())) {
						examAudit(et);
					}
					exportTaskService.endExportTask(et.getId());
				} finally {
					redisClient.delete(cacheLock);
				}
				log.info("ExportTask end");
			} catch (Exception e) {
				log.error("ExportTask error " + e.getMessage(), e);
			}
		}
	}

	/**
	 * 监考已审
	 *
	 * @param et
	 */
	private void examAudit(ExportTaskEntity et) {
		ExamAuditQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamAuditQuery.class);
		req.setRootOrgId(et.getRootOrgId());
		asyncExportService.asyncExportExamAuditList(et.getId(), req);
	}

	/**
	 * 考试明细
	 *
	 * @param et
	 */
	private void examRecordDetails(ExportTaskEntity et) {
		ExamRecordQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamRecordQuery.class);
		req.setRootOrgId(et.getRootOrgId());
		asyncExportService.asyncExportExamRecordDetails(et.getId(), req);
	}

	/**
	 * 成绩统计
	 *
	 * @param et
	 */
	private void examScores(ExportTaskEntity et) {
		ExamScoreQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamScoreQuery.class);
		req.setRootOrgId(et.getRootOrgId());
		asyncExportService.asyncExportExamScoreStatistics(et.getId(), req);
	}

	/**
	 * 考试进度
	 *
	 * @param et
	 */
	private void examScheduling(ExportTaskEntity et) {
		ExamStudentQuery req = new JsonMapper().parseJson(et.getExportParam(), ExamStudentQuery.class);
		req.setRootOrgId(et.getRootOrgId());
		asyncExportService.asyncExportExamScheduling(et.getId(), req);
	}
}
