package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.core.oe.admin.base.Constants;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.NewQuestionType;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Sentence;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordPaperStructRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordPaperStructEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordPaperStructService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.ExamRecordQuestionsInfo;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.BasePaperCacheBean;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 
 * @author  	chenken
 * @date    	2018年11月13日 上午11:52:02
 * @company 	QMTH
 * @description 试卷结构服务类
 */
@Service("examRecordPaperStructService")
public class ExamRecordPaperStructServiceImpl implements ExamRecordPaperStructService{

	@Autowired
	private ExamRecordDataRepo examRecordDataRepo;

	@Autowired
	private ExamRecordPaperStructRepo examRecordPaperStructRepo;
	
	@Autowired
	private JdbcTemplate jdbcTemplate;
	
	@Autowired
	private MongoTemplate mongoTemplate;

	@Override
	public ExamRecordPaperStructEntity getExamRecordPaperStruct(Long examRecordDataId) {
		ExamRecordDataEntity examRecordDataEntity = GlobalHelper.getEntity(examRecordDataRepo,examRecordDataId,ExamRecordDataEntity.class);
		String paperStructId = examRecordDataEntity.getPaperStructId();
		ExamRecordPaperStructEntity examRecordPaperStruct = null;
		if(StringUtils.isBlank(paperStructId)){
			Map<String,Object> paperInfoMap = getPaperInfoFromOeExamQuestion(examRecordDataEntity.getId());
			String paperId = paperInfoMap.get("paper_id").toString();
			//从缓存中获取试卷结构
			BasePaperCacheBean getBasePaperResp = CacheHelper.getBasePaper(paperId);

			examRecordPaperStruct = new ExamRecordPaperStructEntity();
			examRecordPaperStruct.setDefaultPaper(getBasePaperResp.getDefaultPaper());
			//保存作答记录
			List<ExamRecordQuestionsInfo> examRecordQuestionList = encapsulationExamQuestion(examRecordDataEntity);
			if (examRecordQuestionList.size() > 0) {
	             mongoTemplate.insert(examRecordQuestionList, Constants.EXAM_RECORD_QUESTIONS);
	        }
			//保存试卷结构
			examRecordPaperStruct = examRecordPaperStructRepo.save(examRecordPaperStruct);
			//保存考试记录
			//经测试,从oe_exam_question中获取的paperId最准确
			examRecordDataEntity.setBasePaperId(paperId);
			examRecordDataEntity.setPaperType(paperInfoMap.get("paper_type").toString());
			examRecordDataEntity.setPaperStructId(examRecordPaperStruct.getId());
			examRecordDataRepo.save(examRecordDataEntity);
			return examRecordPaperStruct;
		}else{
			examRecordPaperStruct =GlobalHelper.getEntity(examRecordPaperStructRepo,paperStructId,ExamRecordPaperStructEntity.class);
			return examRecordPaperStruct;
		}
	}

	/**
	 * 使用examRecordId查询paperId
	 * @param examRecordId
	 * @return
	 */
	private Map<String,Object> getPaperInfoFromOeExamQuestion(Long examRecordId){
		String sql = "select paper_id,paper_type from oe_exam_question where exam_record_id = "+examRecordId+" limit 1";
		Map<String,Object> examQuestionMap = jdbcTemplate.queryForMap(sql);
		return examQuestionMap;
	}
	
	/**
	 * 封装ExamQuestion
	 * @param examRecordDataEntity
	 */
	private List<ExamRecordQuestionsInfo> encapsulationExamQuestion(ExamRecordDataEntity examRecordDataEntity){
		 List<ExamRecordQuestionsInfo> examRecordQuestionList = new ArrayList<ExamRecordQuestionsInfo>();
		 ExamRecordQuestionsInfo examRecordQuestion = new ExamRecordQuestionsInfo();
         examRecordQuestion.setExamRecordDataId(examRecordDataEntity.getId());
         examRecordQuestion.setCreationTime(examRecordDataEntity.getStartTime());
         List<Map<String, Object>> rows = jdbcTemplate.queryForList(selectOldRecordQuestionByExamRecordIdSql(examRecordDataEntity.getId()));
         for (Map<String, Object> row : rows) {
             ExamRecordQuestionsInfo.ExamQuestionEntity examQuestion = ExamRecordQuestionsInfo.createExamQuestionEntity();

             //转换试题型
             NewQuestionType questionType = NewQuestionType.convert(Sentence.dealString(row.get("question_type")));
             if (questionType == null || questionType == NewQuestionType.NESTED) {
                 continue;
             }

             //获取试题ID
             String questionId = Sentence.dealString(row.get("question_id"));
             if (questionId == null || "".equals(questionId)) {
                 questionId = Sentence.dealString(row.get("parent_question_id"));
             }

             //计算考生试题得分
             Float questionScore = Sentence.dealFloat(row.get("score"));
             String correctAnswer = Sentence.dealString(row.get("answer"));
             String studentAnswer = Sentence.dealString(row.get("stu_answer"));
             Float studentScore = computeStudentScore(questionType, questionScore, correctAnswer, studentAnswer);

             examQuestion.setQuestionId(questionId);
             examQuestion.setMainNumber(Sentence.dealInteger(row.get("main_number")));
             examQuestion.setOrder(Sentence.dealInteger(row.get("orders")));
             examQuestion.setQuestionScore(questionScore);
             examQuestion.setQuestionType(questionType);
             examQuestion.setCorrectAnswer(correctAnswer);
             examQuestion.setStudentAnswer(studentAnswer);
             examQuestion.setStudentScore(studentScore);
             examQuestion.setAnswer(Sentence.dealBoolean(row.get("is_answered")));
             examQuestion.setSign(Sentence.dealBoolean(row.get("is_sign")));
             examQuestion.setOptionPermutation(new Integer[]{0,1,2,3});
             examRecordQuestion.addExamQuestion(examQuestion);
         }
         examRecordQuestionList.add(examRecordQuestion);
         return examRecordQuestionList;
	}
	
	private String selectOldRecordQuestionByExamRecordIdSql(Long examRecordId) {
        return new StringBuilder().append("SELECT ")
                .append("id,exam_record_id,is_answered,is_multiple,is_nested_question,is_sign,orders,")
                .append("main_number,sub_number,base_paper_id,paper_id,paper_name,paper_number,paper_type,")
                .append("parent_question_id,question_id,question_type,score,stu_score,answer,stu_answer,create_at")
                .append(" FROM ").append(Constants.OE_EXAM_QUESTION)
                .append(" WHERE exam_record_id = ").append(examRecordId)
                .append(" ORDER BY orders ASC").toString();
    }
	
    private Float computeStudentScore(NewQuestionType questionType, Float questionScore, String correctAnswer, String studentAnswer) {
        //若为客观题重新计算考生试题得分
        if (questionType == null || !questionType.isObjective()) {
            return 0F;
        }
        if (studentAnswer == null || "".equals(studentAnswer)) {
            return 0F;
        }

        //按字母顺序排序  D,A,C --> ACD
        String[] answers = studentAnswer.split(",");
        Arrays.sort(answers);
        String answerStr = new String();
        for (String answer : answers) {
            answerStr += answer;
        }

        if (!answerStr.equals(correctAnswer)) {
            return 0F;
        }
        return questionScore;
    }
	
}
