/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-31 09:33:22.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.core.oe.admin.base.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.CommonUtil;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStudentFinalScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.CourseLevel;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentFinalScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.admin.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreEntityConvert;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ObjectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;

/**
 * 考试分数相关接口
 *
 * @author: QMTH
 * @since: 2018/8/29
 */
@Service
public class ExamScoreServiceImpl implements ExamScoreService {
    @Autowired
    private ExamScoreRepo examScoreRepo;
    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;
    @Autowired
    private ExamStudentService examStudentService;
    @Autowired
    private ExamRecordService examRecordService;
    @Autowired
    private GainBaseDataService gainBaseDataService;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ExamStudentFinalScoreService examStudentFinalScoreService;

    @Override
    public Page<ExamScoreInfo> getExamScoreList(ExamScoreQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试批次！");
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize());


        //阅卷方式
        String markingType = ExamCacheTransferHelper.
                getDefaultCachedExamProperty(query.getExamId(), ExamProperties.MARKING_TYPE.name()).getValue();

        //获取考生列表
        Page<ExamStudentInfo> page = examStudentService.getExamStudentListPage(ExamScoreEntityConvert.of(query));
        List<ExamStudentInfo> examStudentList = page.getContent();
        if (examStudentList == null || examStudentList.size() == 0) {
            return new PageImpl<ExamScoreInfo>(Lists.newArrayList(), pageable, page.getTotalElements());
        }

        //封装成绩统计结果
        List<ExamScoreInfo> examScoreList = new ArrayList<>();
        examStudentList.forEach(examStudent -> {
            //获取考试信息
            ExamSettingsCacheBean examBean = ExamCacheTransferHelper.getDefaultCachedExam(query.getExamId());
            examScoreList.add(of(examStudent, examBean, markingType));
        });
        return new PageImpl<ExamScoreInfo>(examScoreList, pageable, page.getTotalElements());
    }

    /**
     * 封装成绩统计结果
     */
    private ExamScoreInfo of(ExamStudentInfo examStudent, ExamSettingsCacheBean examBean, String markingType) {
        ExamScoreInfo examScore = new ExamScoreInfo();
        examScore.setExamId(examStudent.getExamId());
        examScore.setExamName(examBean.getName());
        examScore.setStudentId(examStudent.getStudentId());
        examScore.setStudentCode(examStudent.getStudentCode());
        examScore.setStudentName(examStudent.getStudentName());
        examScore.setIdentityNumber(examStudent.getIdentityNumber());

        examScore.setCourseId(examStudent.getCourseId());
        examScore.setCourseCode(examStudent.getCourseCode());
        examScore.setCourseName(examStudent.getCourseName());
        examScore.setCourseNameAndCode(examStudent.getCourseName() + "(" + examStudent.getCourseCode() + ")");
        examScore.setCourseLevel(CourseLevel.getCourseLevelTitle(examStudent.getCourseLevel()));

        examScore.setOrgId(examStudent.getOrgId());
        examScore.setOrgCode(examStudent.getOrgCode());
        examScore.setOrgName(examStudent.getOrgName());
        examScore.setInfoCollector(examStudent.getInfoCollector());
        examScore.setSpecialtyName(examStudent.getSpecialtyName());
        examScore.setGrade(examStudent.getGrade());
        Long examTimes = examBean.getExamTimes();
        if (examTimes != null) {
            Integer extraExamNum = examStudent.getExtraNum() == null ? 0 : examStudent.getExtraNum();
            Integer usedNum = examStudent.getUsedNum() == null ? 0 : examStudent.getUsedNum();
            Long leftExamTimes = (examTimes + extraExamNum - usedNum) < 0 ? 0 : (examTimes + extraExamNum - usedNum);
            examScore.setLeftExamTimes(leftExamTimes);
        }
        return setCommomScoreInfo(examScore, examStudent.getFinished(), examStudent.getExamStudentId(), examBean, markingType);
    }

    @Override
    public List<ExamScoreInfo> exportExamScoreListForAsync(ExamScoreQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试批次！");

        //阅卷方式
        String markingType = ExamCacheTransferHelper.
                getDefaultCachedExamProperty(query.getExamId(), ExamProperties.MARKING_TYPE.name()).getValue();
        if (query.getStartLimit() != null && query.getEndLimit() != null) {
        	return getExamStudentInfoListOfScoreExport(query, markingType);
        }else {
        	List<ExamStudentEntity> examStudentList=new ArrayList<ExamStudentEntity>();
            Long startId=0L;
            for(;;) {
            	List<ExamStudentEntity> tem=getExamStudentInfoListOfScoreExportByPage(query, markingType, startId);
            	if(tem==null||tem.size()==0) {
            		break;
            	}else {
            		startId=tem.get(tem.size()-1).getId();
            		examStudentList.addAll(tem);
            	}
            }
            
            List<ExamScoreInfo> examScoreInfoList = new ArrayList<ExamScoreInfo>();
            //缓存
            for (ExamStudentEntity examStudent : examStudentList) {
                long courseId = examStudent.getCourseId();
                CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);

                long orgId = examStudent.getOrgId();
                OrgCacheBean orgBean = gainBaseDataService.getOrgBean(orgId);

                examScoreInfoList.add(convertToExamScoreInfo(examStudent, courseBean, orgBean, markingType));
            }
            return examScoreInfoList;
        }
    }
    
    private List<ExamStudentEntity> getExamStudentInfoListOfScoreExportByPage(ExamScoreQuery query, String markingType,Long startId) {
        //查询条件
        StringBuffer sql = new StringBuffer();
        sql.append("select id,exam_student_id,exam_id,course_id,course_code,course_level,finished,student_id,student_code,student_name,identity_number"
                + ",info_collector,root_org_id,org_id,paper_type,used_num,extra_num"
                + ",specialty_code,specialty_name,grade from ec_oe_exam_student where 1=1 ");
        if (query.getOrgId() != null) {
            sql.append(" and org_id=" + query.getOrgId());
        }
        if (query.getExamId() != null) {
            sql.append(" and exam_id=" + query.getExamId());
        }
        if (query.getExamStageId() != null) {
            sql.append(" and exam_stage_id=" + query.getExamStageId());
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and student_code LIKE '%" + query.getStudentCode() + "%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and student_name LIKE '%" + query.getStudentName() + "%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and identity_number LIKE '" + query.getIdentityNumber() + "%'");
        }
        if (query.getCourseId() != null) {
            sql.append(" and course_id=" + query.getCourseId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and course_level= '" + query.getCourseLevel() + "'");
        }
        if (query.getFinished() != null) {
            sql.append(" and finished= " + query.getFinished());
        }
        sql.append(" and id >"+startId);
        sql.append(" order by id limit 500 ");

        List<ExamStudentEntity> examStudentList = jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudentEntity>() {
            @Override
            public ExamStudentEntity mapRow(ResultSet rs, int rowNum) throws SQLException {
                return getExamStudentEntityByResultSet(rs);
            }
        });

        return examStudentList;
    }
    
    @Override
    public List<ExamScoreInfo> exportExamScoreList(ExamScoreQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试批次！");

        //阅卷方式
        String markingType = ExamCacheTransferHelper.
                getDefaultCachedExamProperty(query.getExamId(), ExamProperties.MARKING_TYPE.name()).getValue();
        return getExamStudentInfoListOfScoreExport(query, markingType);
    }

    @Override
    public List<ExamScoreEntity> getAllExamScoreList(Long examId, String identityNumber, Long courseId) {
        List<ExamRecordDataEntity> allExamRecordDataList =
                examRecordDataRepo.findByExamIdAndIdentityNumberAndCourseId(examId, identityNumber, courseId);
        if (allExamRecordDataList == null || allExamRecordDataList.isEmpty()) {
            return null;
        }

        List<Long> examRecordDataIds = allExamRecordDataList.stream().map(p -> p.getId()).collect(Collectors.toList());
        return examScoreRepo.findByExamRecordDataIdIn(examRecordDataIds);
    }

    private List<ExamScoreInfo> getExamStudentInfoListOfScoreExport(ExamScoreQuery query, String markingType) {
        Check.isNull(query, "查询参数不能为空！");
        //查询条件
        StringBuffer sql = new StringBuffer();
        sql.append("select id,exam_student_id,exam_id,course_id,course_code,course_level,finished,student_id,student_code,student_name,identity_number"
                + ",info_collector,root_org_id,org_id,paper_type,used_num,extra_num"
                + ",specialty_code,specialty_name,grade from ec_oe_exam_student where 1=1 ");
        if (query.getOrgId() != null) {
            sql.append(" and org_id=" + query.getOrgId());
        }
        if (query.getExamId() != null) {
            sql.append(" and exam_id=" + query.getExamId());
        }
        if (query.getExamStageId() != null) {
            sql.append(" and exam_stage_id=" + query.getExamStageId());
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and student_code LIKE '%" + query.getStudentCode() + "%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and student_name LIKE '%" + query.getStudentName() + "%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and identity_number LIKE '" + query.getIdentityNumber() + "%'");
        }
        if (query.getCourseId() != null) {
            sql.append(" and course_id=" + query.getCourseId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and course_level= '" + query.getCourseLevel() + "'");
        }
        if (query.getFinished() != null) {
            sql.append(" and finished= " + query.getFinished());
        }
        sql.append(" order by update_time desc");

        if (query.getStartLimit() != null && query.getEndLimit() != null) {
            int offset = query.getEndLimit() - query.getStartLimit() + 1;
            sql.append(" limit " + (query.getStartLimit() - 1) + "," + offset);
        }
        List<ExamStudentEntity> examStudentList = jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudentEntity>() {
            @Override
            public ExamStudentEntity mapRow(ResultSet rs, int rowNum) throws SQLException {
                return getExamStudentEntityByResultSet(rs);
            }
        });

        List<ExamScoreInfo> examScoreInfoList = new ArrayList<ExamScoreInfo>();
        //缓存
        for (ExamStudentEntity examStudent : examStudentList) {
            long courseId = examStudent.getCourseId();
            CourseCacheBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);

            long orgId = examStudent.getOrgId();
            OrgCacheBean orgBean = gainBaseDataService.getOrgBean(orgId);

            examScoreInfoList.add(convertToExamScoreInfo(examStudent, courseBean, orgBean, markingType));
        }
        return examScoreInfoList;
    }

    private ExamScoreInfo convertToExamScoreInfo(ExamStudentEntity examStudent, CourseCacheBean courseBean, OrgCacheBean orgBean, String markingType) {
        if (examStudent == null) {
            return null;
        }
        //获取考试信息
        ExamSettingsCacheBean examBean = ExamCacheTransferHelper.getCachedExam(examStudent.getExamId(),
                examStudent.getStudentId(), examStudent.getExamStageId());
        ExamScoreInfo examScore = new ExamScoreInfo();
        examScore.setExamId(examStudent.getExamId());
        examScore.setExamName(examBean.getName());
        examScore.setStudentId(examStudent.getStudentId());
        examScore.setStudentCode(examStudent.getStudentCode());
        examScore.setStudentName(examStudent.getStudentName());
        examScore.setIdentityNumber(examStudent.getIdentityNumber());
        examScore.setInfoCollector(examStudent.getInfoCollector());

        examScore.setCourseId(examStudent.getCourseId());
        examScore.setCourseCode(examStudent.getCourseCode());
        examScore.setCourseName(courseBean.getName());
        examScore.setCourseNameAndCode(courseBean.getName() + "(" + courseBean.getCode() + ")");
        examScore.setCourseLevel(CourseLevel.getCourseLevelTitle(examStudent.getCourseLevel()));

        examScore.setOrgId(examStudent.getOrgId());
        examScore.setOrgCode(orgBean.getCode());
        examScore.setOrgName(orgBean.getName());

        examScore.setSpecialtyName(examStudent.getSpecialtyName());
        examScore.setGrade(examStudent.getGrade());

        //剩余考试次数=可考次数+可重考次数-已考次数
        Long examTimes = examBean.getExamTimes();
        if (examTimes != null) {
            Integer extraExamNum = examStudent.getExtraNum() == null ? 0 : examStudent.getExtraNum();
            Integer usedNum = examStudent.getUsedNum() == null ? 0 : examStudent.getUsedNum();
            Long leftExamTimes = (examTimes + extraExamNum - usedNum) < 0 ? 0 : (examTimes + extraExamNum - usedNum);
            examScore.setLeftExamTimes(leftExamTimes);
        }

        return setCommomScoreInfo(examScore, examStudent.getFinished(), examStudent.getExamStudentId(), examBean, markingType);
    }

    private ExamScoreInfo setCommomScoreInfo(ExamScoreInfo examScoreInfo, Boolean finished, long examStudentId, ExamSettingsCacheBean examBean, String markingType) {
        //查询考试记录
        if (finished != null && finished) {
            //查询考试记录
            List<ExamRecordDataEntity> examRecordList = examRecordService.getExamRecordListByExamStudentId(examStudentId);
            examScoreInfo.setIsFinished("是");//是否完成考试
            examScoreInfo.setIsAbsent("否");//是否缺考
            //违纪次数
            examScoreInfo.setDisciplineCount(disciplineCount(examRecordList));
            //提交次数
            examScoreInfo.setSubmitCount(submitCount(examRecordList));

            //设置相关成绩（总分，主观分，客观分）
            ExamStudentFinalScoreEntity finalExamScore = examStudentFinalScoreService.getFinalEffectiveExamScore(examStudentId);
            setFinalExamScore(examScoreInfo, finalExamScore);

            //需求调整20200923：将开考时间，改为最终成绩的更新时间
            examScoreInfo.setStartTime(getExamStudentFinalScoreUpdateTime(finalExamScore));
        } else {
            examScoreInfo.setIsFinished("否");//是否完成考试
            examScoreInfo.setIsAbsent("是");//是否缺考
            examScoreInfo.setDisciplineCount("--");//违纪次数
            examScoreInfo.setSubmitCount("--"); //提交次数
            examScoreInfo.setObjectiveScore("--");//主观分
            examScoreInfo.setSubjectiveScore("--");//客观分
            examScoreInfo.setFinalExamScore("--");//最终成绩
            examScoreInfo.setStartTime("--");//考试开始时间
        }
        return examScoreInfo;
    }


    /**
     * 给考生最终成绩赋值
     *
     * @param examScoreInfo 考试模型实体
     * @param examScore     考试数据实体
     */
    private void setFinalExamScore(ExamScoreInfo examScoreInfo, final ExamStudentFinalScoreEntity examScore) {
        if (examScore == null) {
            examScoreInfo.setObjectiveScore("0");
            examScoreInfo.setSubjectiveScore("0");
            examScoreInfo.setFinalExamScore("0");
        } else {
            DecimalFormat decimalFormat = new DecimalFormat("#.##");
            examScoreInfo.setObjectiveScore(decimalFormat.format(null == examScore.getObjectiveScore() ? 0 : examScore.getObjectiveScore()));
            examScoreInfo.setSubjectiveScore(decimalFormat.format(null == examScore.getSubjectiveScore() ? 0 : examScore.getSubjectiveScore()));
            examScoreInfo.setFinalExamScore(decimalFormat.format(null == examScore.getTotalScore() ? 0 : examScore.getTotalScore()));
        }
    }


    /**
     * 考试开始时间
     */
    private String getExamStudentFinalScoreUpdateTime(ExamStudentFinalScoreEntity examScore) {
        if (examScore == null || examScore.getUpdateTime() == null) {
            return "";
        }

        return CommonUtil.getDateStrWithSecond(examScore.getUpdateTime());
    }

    /**
     * 违纪次数
     */
    private String disciplineCount(List<ExamRecordDataEntity> examRecordList) {
        Integer disciplineCount = 0;
        if (examRecordList == null) {
            return disciplineCount.toString();
        }
        for (ExamRecordDataEntity e : examRecordList) {
            if (e.getIsIllegality() != null && e.getIsIllegality()) {
                disciplineCount++;
            }
        }
        return disciplineCount.toString();
    }

    /**
     * 正常交卷次数:无警告或者有警告,有审核,有正常提交时间
     */
    private String submitCount(List<ExamRecordDataEntity> examRecordList) {
        Integer submitCount = 0;
        for (ExamRecordDataEntity e : examRecordList) {
            if (e.getIsWarn() == null) {
                e.setIsWarn(false);
            }
            if (e.getIsAudit() == null) {
                e.setIsAudit(false);
            }
            if (e.getIsWarn() && !e.getIsAudit()) {
                //ignore
            } else {
                if (e.getEndTime() != null) {
                    submitCount++;
                }
            }
        }
        return submitCount.toString();
    }

    private ExamStudentEntity getExamStudentEntityByResultSet(ResultSet rs) throws SQLException {
        ExamStudentEntity examStudentEntity = new ExamStudentEntity();
        examStudentEntity.setId(rs.getLong("id"));
        examStudentEntity.setExamStudentId(rs.getLong("exam_student_id"));
        examStudentEntity.setExamId(rs.getLong("exam_id"));
        examStudentEntity.setCourseId(rs.getLong("course_id"));
        examStudentEntity.setCourseCode(rs.getString("course_code"));
        examStudentEntity.setCourseLevel(rs.getString("course_level"));
        examStudentEntity.setFinished(rs.getBoolean("finished"));
        examStudentEntity.setStudentId(rs.getLong("student_id"));
        examStudentEntity.setStudentCode(rs.getString("student_code"));
        examStudentEntity.setStudentName(rs.getString("student_name"));
        examStudentEntity.setIdentityNumber(rs.getString("identity_number"));
        examStudentEntity.setInfoCollector(rs.getString("info_collector"));
        examStudentEntity.setRootOrgId(rs.getLong("root_org_id"));
        examStudentEntity.setOrgId(rs.getLong("org_id"));
        examStudentEntity.setPaperType(rs.getString("paper_type"));
        examStudentEntity.setUsedNum(rs.getInt("used_num"));
        examStudentEntity.setExtraNum(rs.getInt("extra_num"));
        examStudentEntity.setSpecialtyCode(rs.getString("specialty_code"));
        examStudentEntity.setSpecialtyName(rs.getString("specialty_name"));
        examStudentEntity.setGrade(rs.getString("grade"));
        return examStudentEntity;
    }

    @Override
    public void createExamScoreWithOffline(Long examRecordDataId) {
        ExamScoreEntity examScoreEntity = new ExamScoreEntity();
        examScoreEntity.setExamRecordDataId(examRecordDataId);
        examScoreEntity.setObjectiveScore(0D);
        examScoreEntity.setSubjectiveScore(0D);
        examScoreEntity.setTotalScore(0D);
        examScoreRepo.save(examScoreEntity);
    }

    @Override
    public List<ObjectiveScoreInfo> queryObjectiveScoreList(Long examStudentId) {
        List<ExamRecordDataEntity> examRecordDataList = examRecordDataRepo.findByExamStudentId(examStudentId);
        //过滤已完成的考试记录(包括违纪的)
        examRecordDataList = examRecordDataList.stream().filter((o -> {
            return o.getExamRecordStatus() == ExamRecordStatus.EXAM_END ||
                    o.getExamRecordStatus() == ExamRecordStatus.EXAM_OVERDUE ||
                    o.getExamRecordStatus() == ExamRecordStatus.EXAM_HAND_IN ||
                    o.getExamRecordStatus() == ExamRecordStatus.EXAM_AUTO_HAND_IN;
        })).collect(Collectors.toList());

        List<ObjectiveScoreInfo> objectiveScoreInfoList = new ArrayList<ObjectiveScoreInfo>();
        for (ExamRecordDataEntity examRecordDataEntity : examRecordDataList) {
            ObjectiveScoreInfo objectiveScoreInfo = new ObjectiveScoreInfo();
            objectiveScoreInfo.setExamRecordDataId(examRecordDataEntity.getId());
            objectiveScoreInfo.setExamOrder(examRecordDataEntity.getExamOrder());
            objectiveScoreInfo.setStartTime(examRecordDataEntity.getStartTime());
            objectiveScoreInfo.setEndTime(examRecordDataEntity.getEndTime());

            //如果考试没有结束，则只能返回部分数据
            if (!isExamRecordEnded(examRecordDataEntity)) {
                objectiveScoreInfo.setIsExamEnded(false);
                objectiveScoreInfoList.add(objectiveScoreInfo);
                continue;
            } else {
                objectiveScoreInfo.setIsExamEnded(true);
            }

            if (examRecordDataEntity.getIsIllegality() == null || !examRecordDataEntity.getIsIllegality()) {
                if (examRecordDataEntity.getIsWarn() && !examRecordDataEntity.getIsAudit()) {
                    objectiveScoreInfo.setIsAuditing(true);
                } else if (!examRecordDataEntity.getIsWarn() || (examRecordDataEntity.getIsWarn() && examRecordDataEntity.getIsAudit())) {
                    ExamScoreEntity examScore = examScoreRepo.findByExamRecordDataId(examRecordDataEntity.getId());
                    objectiveScoreInfo.setIsAuditing(false);
                    objectiveScoreInfo.setObjectiveScore(examScore.getObjectiveScore());
                }
                objectiveScoreInfo.setIsIllegality(false);
            } else {
                objectiveScoreInfo.setIsIllegality(true);
            }
            objectiveScoreInfoList.add(objectiveScoreInfo);
        }
        return objectiveScoreInfoList;
    }

    private boolean isExamRecordEnded(ExamRecordDataEntity examRecordData) {
        //如果考试记录状态为已处理，则直接返回true.
        if (examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_END ||
                examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_OVERDUE ||
                examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_INVALID) {
            return true;
        }
        return false;
    }
}
